<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:redirect="http://xml.apache.org/xalan/redirect"
    xmlns:sablecc="sablecc-extension"
    extension-element-prefixes="redirect sablecc">

  <xsl:output method="text"/>

  <xalan:component prefix="sablecc" functions="escape_string_literal">
    <xalan:script lang="javascript">
      // Escape a unicode string into java edible string literal
      function escape_string_literal(val)
      {
        var ret = '';
        for ( var i = 0; i &lt; val.length; i++ ) {
            if ( val.charAt(i) == '\n' ) {
                ret += '\\n';
            } else if ( val.charAt(i) == '\r' ) {
                ret += '\\r';
            } else if ( val.charAt(i) == '"' ) {
                ret += '\\"';
            } else if ( val.charAt(i) == '\\' ) {
                ret += '\\\\';
            } else if ( val.charCodeAt(i) == 0 ) {
                ret += '\\0';
            } else if ( val.charCodeAt(i) &lt; 0x20 || val.charCodeAt(i) &gt; 0x7f ||
                val.charAt(i) == '"' || val.charAt(i) == '\\') {
                var str = val.charCodeAt(i).toString(16);
                while ( str.length &lt; 4 ) str = '0' + str;
                ret += '\\u' + str;
            } else {
                ret += val.charAt(i);
            }
        }
        return ret;
      }
    </xalan:script>
  </xalan:component>

  <xsl:param name="basedir" select="'.'"/>
  <xsl:param name="newline" select="'&#10;'"/>

  <!-- include all components -->
  <xsl:include href="make_nodes.xslt"/>
  <xsl:include href="make_tokens.xslt"/>
  <xsl:include href="make_prods.xslt"/>
  <xsl:include href="make_analysis.xslt"/>
  <xsl:include href="make_lexer.xslt"/>
  <xsl:include href="make_parser.xslt"/>
  <xsl:include href="make_utils.xslt"/>

  <!-- match the root node 'parser' -->
  <xsl:template match="parser">
    <xsl:call-template name="make_nodes"/>
    <xsl:call-template name="make_tokens"/>
    <xsl:call-template name="make_prods"/>
    <xsl:call-template name="make_analysis"/>
    <xsl:call-template name="make_lexer"/>
    <xsl:call-template name="make_parser"/>
    <xsl:call-template name="make_utils"/>
  </xsl:template>

</xsl:stylesheet>

