<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:redirect="http://xml.apache.org/xalan/redirect"
    xmlns:sablecc="sablecc-extension"
    extension-element-prefixes="redirect sablecc">

  <xsl:template name="make_lexer">

    <redirect:write select="concat($basedir,'/lexer.cs')" mkdirs="true">
/* This file was generated by SableCC (http://www.sablecc.org/). */

using System;
using System.Collections;
using System.Text;
using System.IO;
using <xsl:value-of select="/parser/@package"/>.node;

namespace <xsl:value-of select="/parser/@package"/>.lexer {


internal class PushbackReader {
  private TextReader reader;
  private Stack stack = new Stack ();


  internal PushbackReader (TextReader reader)
  {
    this.reader = reader;
  }

  internal int Peek ()
  {
    if ( stack.Count > 0 ) return (int)stack.Peek();
    return reader.Peek();
  }

  internal int Read ()
  {
    if ( stack.Count > 0 ) return (int)stack.Pop();
    return reader.Read();
  }

  internal void Unread (int v)
  {
    stack.Push (v);
  }
}

    <!-- generate the LexerException class -->
public class LexerException : ApplicationException
{
    public LexerException(String message) : base (message)
    {
    }
}

    <!-- generate the Lexer class -->

public class Lexer
{
    protected Token token;
    protected State state = State.<xsl:value-of select="lexer_data/state/@name"/>;

    private PushbackReader input;
    private int line;
    private int pos;
    private bool cr;
    private bool eof;
    private StringBuilder text = new StringBuilder();

    protected virtual void Filter()
    {
    }

    public Lexer(TextReader input)
    {
        this.input = new PushbackReader(input);
    }

    public virtual Token Peek()
    {
        while(token == null)
        {
            token = GetToken();
            Filter();
        }

        return token;
    }

    public virtual Token Next()
    {
        while(token == null)
        {
            token = GetToken();
            Filter();
        }

        Token result = token;
        token = null;
        return result;
    }

    protected virtual Token GetToken()
    {
        int dfa_state = 0;

        int start_pos = pos;
        int start_line = line;

        int accept_state = -1;
        int accept_token = -1;
        int accept_length = -1;
        int accept_pos = -1;
        int accept_line = -1;

        int[][][] gotoTable = Lexer.gotoTable[state.id()];
        int[] accept = Lexer.accept[state.id()];
        text.Length = 0;

        while(true)
        {
            int c = GetChar();

            if(c != -1)
            {
                switch(c)
                {
                case 10:
                    if(cr)
                    {
                        cr = false;
                    }
                    else
                    {
                        line++;
                        pos = 0;
                    }
                    break;
                case 13:
                    line++;
                    pos = 0;
                    cr = true;
                    break;
                default:
                    pos++;
                    cr = false;
                    break;
                };

                text.Append((char) c);
                do
                {
                    int oldState = (dfa_state &lt; -1) ? (-2 -dfa_state) : dfa_state;

                    dfa_state = -1;

                    int[][] tmp1 =  gotoTable[oldState];
                    int low = 0;
                    int high = tmp1.Length - 1;

                    while(low &lt;= high)
                    {
                        int middle = (low + high) / 2;
                        int[] tmp2 = tmp1[middle];

                        if(c &lt; tmp2[0])
                        {
                            high = middle - 1;
                        }
                        else if(c &gt; tmp2[1])
                        {
                            low = middle + 1;
                        }
                        else
                        {
                            dfa_state = tmp2[2];
                            break;
                        }
                    }
                }while(dfa_state &lt; -1);
            }
            else
            {
                dfa_state = -1;
            }

            if(dfa_state &gt;= 0)
            {
                if(accept[dfa_state] != -1)
                {
                    accept_state = dfa_state;
                    accept_token = accept[dfa_state];
                    accept_length = text.Length;
                    accept_pos = pos;
                    accept_line = line;
                }
            }
            else
            {
                if(accept_state != -1)
                {
                    switch(accept_token)
                    {
<xsl:for-each select="tokens/token">
                    case <xsl:value-of select="position()-1"/>:
                        {
<xsl:if test="not(@value)">
                            Token token = New<xsl:value-of select="position()-1"/>(
                                GetText(accept_length),
                                start_line + 1,
                                start_pos + 1);
</xsl:if>
<xsl:if test="@value">
                            Token token = New<xsl:value-of select="position()-1"/>(
                                start_line + 1,
                                start_pos + 1);
</xsl:if>
                            PushBack(accept_length);
                            pos = accept_pos;
                            line = accept_line;
<xsl:if test="count(transition[@from!=@to])!=0">
                            switch (state.id())
                            {
<xsl:for-each select="transition[@from!=@to]">
                              <xsl:variable name="crossref" select="@from"/>
                              case <xsl:value-of select="/parser/lexer_data/state[@name=$crossref]/@id"/>: state = State.<xsl:value-of select="@to"/>; break;
</xsl:for-each>
                            }
</xsl:if>
                            return token;
                        }
</xsl:for-each>
                    }
                }
                else
                {
                    if(text.Length &gt; 0)
                    {
                        throw new LexerException(
                            "[" + (start_line + 1) + "," + (start_pos + 1) + "]" +
                            " Unknown token: " + text);
                    }
                    else
                    {
                        EOF token = new EOF(
                            start_line + 1,
                            start_pos + 1);
                        return token;
                    }
                }
            }
        }
    }

<xsl:for-each select="tokens/token">
<xsl:if test="@value">
    private Token New<xsl:value-of select="position()-1"/>(int line, int pos) { return new <xsl:value-of select="@ename"/>(line, pos); }
</xsl:if>
<xsl:if test="not(@value)">
    private Token New<xsl:value-of select="position()-1"/>(String text, int line, int pos) { return new <xsl:value-of select="@ename"/>(text, line, pos); }
</xsl:if>
</xsl:for-each>

    private int GetChar()
    {
        if(eof)
        {
            return -1;
        }

        int result = input.Read();

        if(result == -1)
        {
            eof = true;
        }

        return result;
    }

    private void PushBack(int acceptLength)
    {
        int length = text.Length;
        for(int i = length - 1; i &gt;= acceptLength; i--)
        {
            eof = false;

            input.Unread(text[i]);
        }
    }


    protected virtual void Unread(Token token)
    {
        String text = token.Text;
        int length = text.Length;

        for(int i = length - 1; i &gt;= 0; i--)
        {
            eof = false;

            input.Unread(text[i]);
        }

        pos = token.Pos - 1;
        line = token.Line - 1;
    }

    private string GetText(int acceptLength)
    {
        StringBuilder s = new StringBuilder(acceptLength);
        for(int i = 0; i &lt; acceptLength; i++)
        {
            s.Append(text[i]);
        }

        return s.ToString();
    }

    private static int[][][][] gotoTable = {
<xsl:for-each select="lexer_data/goto_table/state">
  <xsl:text>new int[][][] {</xsl:text>
  <xsl:for-each select="row">
    <xsl:text>new int[][] {</xsl:text>
    <xsl:for-each select="goto">
        <xsl:value-of select="concat('new int[] {',@low,', ', @high, ', ', @state, '}')"/>
        <xsl:if test="position()!=last()">, </xsl:if>
    </xsl:for-each>
    <xsl:text>}</xsl:text><xsl:if test="position()!=last()">,
    </xsl:if>
  </xsl:for-each>
  <xsl:text>}</xsl:text><xsl:if test="position()!=last()">,
  </xsl:if>
</xsl:for-each>
    };

    private static int[][] accept = {
<xsl:for-each select="lexer_data/accept_table/state">
    new int[] {
<xsl:for-each select="i">
  <xsl:value-of select="."/><xsl:if test="position()!=last()">, </xsl:if>
</xsl:for-each>
    }<xsl:if test="position()!=last()">, </xsl:if>
</xsl:for-each>
    };

    public class State
    {
<xsl:for-each select="lexer_data/state">
        public static State <xsl:value-of select="@name"/> = new State(<xsl:value-of select="@id"/>);
</xsl:for-each>

        private int _id;

        private State(int _id)
        {
            this._id = _id;
        }

        public int id()
        {
            return _id;
        }
    }
}
}
    </redirect:write>
  </xsl:template>

</xsl:stylesheet>

