<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:redirect="http://xml.apache.org/xalan/redirect"
    xmlns:sablecc="sablecc-extension"
    extension-element-prefixes="redirect sablecc">

  <xsl:template name="make_parser">

    <redirect:write select="concat($basedir,'/parser.cs')" mkdirs="true">
/* This file was generated by SableCC (http://www.sablecc.org/). */

using System;
using System.Collections;
using System.Text;
using System.IO;
using <xsl:value-of select="/parser/@package"/>.node;
using <xsl:value-of select="/parser/@package"/>.lexer;
using <xsl:value-of select="/parser/@package"/>.analysis;

namespace <xsl:value-of select="/parser/@package"/>.parser {

    <!-- generate the ParserException class -->

public class ParserException : ApplicationException
{
    Token token;

    public ParserException(Token token, String  message) : base(message)
    {
        this.token = token;
    }

    public Token Token
    {
      get { return token; }
    }
}

    <!-- generate the State class -->
internal sealed class State
{
    internal int state;
    internal ArrayList nodes;

    internal State(int state, ArrayList nodes)
    {
        this.state = state;
        this.nodes = nodes;
    }
}

    <!-- generate the TokenIndex class -->

internal class TokenIndex : AnalysisAdapter
{
    internal int index;
<xsl:for-each select="tokens/token[@parser_index]">
    public override void Case<xsl:value-of select="@ename"/>(<xsl:value-of select="@ename"/> node)
    {
        index = <xsl:value-of select="@parser_index"/>;
    }
</xsl:for-each>

    public override void CaseEOF(EOF node)
    {
        index = <xsl:value-of select="tokens/eof/@parser_index"/>;
    }
}

    <!-- generate the Parser class -->

public class Parser
{
    public /* XXX final */ Analysis ignoredTokens = new AnalysisAdapter();

    protected ArrayList nodeList;

    private Lexer lexer;
    private Stack stack = new Stack();
    private int last_shift;
    private int last_pos;
    private int last_line;
    private Token last_token;
    private TokenIndex converter = new TokenIndex();
    private int[] action = new int[2];

    private const int SHIFT = 0;
    private const int REDUCE = 1;
    private const int ACCEPT = 2;
    private const int ERROR = 3;

    public Parser(Lexer lexer)
    {
        this.lexer = lexer;
    }

    private int GoTo(int index)
    {
        int state = State();
        int low = 1;
        int high = gotoTable[index].Length - 1;
        int value = gotoTable[index][0][1];

        while(low &lt;= high)
        {
            int middle = (low + high) / 2;

            if(state &lt; gotoTable[index][middle][0])
            {
                high = middle - 1;
            }
            else if(state &gt; gotoTable[index][middle][0])
            {
                low = middle + 1;
            }
            else
            {
                value = gotoTable[index][middle][1];
                break;
            }
        }

        return value;
    }

    private void Push(int numstate, ArrayList listNode)
    {
        this.nodeList = listNode;

        stack.Push(new State(numstate, this.nodeList));
    }

    private int State()
    {
        State s = (State) stack.Peek();
        return s.state;
    }

    private ArrayList Pop()
    {
        return (ArrayList) ((State) stack.Pop()).nodes;
    }

    private int Index(Switchable token)
    {
        converter.index = -1;
        token.Apply(converter);
        return converter.index;
    }

    public Start Parse()
    {
        Push(0, null);

        IList ign = null;
        while(true)
        {
            while(Index(lexer.Peek()) == -1)
            {
                if(ign == null)
                {
                    ign = new TypedList(NodeCast.Instance);
                }

                ign.Add(lexer.Next());
            }

            if(ign != null)
            {
                ignoredTokens.SetIn(lexer.Peek(), ign);
                ign = null;
            }

            last_pos = lexer.Peek().Pos;
            last_line = lexer.Peek().Line;
            last_token = lexer.Peek();

            int index = Index(lexer.Peek());
            action[0] = actionTable[State()][0][1];
            action[1] = actionTable[State()][0][2];

            int low = 1;
            int high = actionTable[State()].Length - 1;

            while(low &lt;= high)
            {
                int middle = (low + high) / 2;

                if(index &lt; actionTable[State()][middle][0])
                {
                    high = middle - 1;
                }
                else if(index &gt; actionTable[State()][middle][0])
                {
                    low = middle + 1;
                }
                else
                {
                    action[0] = actionTable[State()][middle][1];
                    action[1] = actionTable[State()][middle][2];
                    break;
                }
            }

            switch(action[0])
            {
                case SHIFT:
        {
            ArrayList list = new ArrayList();
            list.Add(lexer.Next());
                        Push(action[1], list);
                        last_shift = action[1];
                    }
        break;
                case REDUCE:
                    switch(action[1])
                    {
<xsl:for-each select="rules/rule">
                    case <xsl:value-of select="@index"/>:
        {
      ArrayList list = New<xsl:value-of select="@index"/>();
      Push(GoTo(<xsl:value-of select="@leftside"/>), list);
        }
        break;
</xsl:for-each>
                    }
                    break;
                case ACCEPT:
                    {
                        EOF node2 = (EOF) lexer.Next();
                        <xsl:value-of select="/parser/productions/production/@ename"/> node1 = (<xsl:value-of select="/parser/productions/production/@ename"/>) ((ArrayList)Pop())[0];
                        Start node = new Start(node1, node2);
                        return node;
                    }
                case ERROR:
                    throw new ParserException(last_token,
                        "[" + last_line + "," + last_pos + "] " +
                        errorMessages[errors[action[1]]]);
            }
        }
    }

<xsl:for-each select="rules/rule">
    ArrayList New<xsl:value-of select="@index"/>()
    {
        ArrayList nodeList = new ArrayList();
<xsl:for-each select="action">
  <xsl:choose>
    <xsl:when test="@cmd='POP'">
        ArrayList <xsl:value-of select="@result"/> = (ArrayList) Pop();
    </xsl:when>
    <xsl:when test="@cmd='FETCHLIST'">
        TypedList <xsl:value-of select="@result"/> = (TypedList)<xsl:value-of select="@from"/>[<xsl:value-of select="@index"/>];
    </xsl:when>
    <xsl:when test="@cmd='FETCHNODE'">
        <xsl:value-of select="@etype"/><xsl:text> </xsl:text><xsl:value-of select="@result"/> = (<xsl:value-of select="@etype"/>)<xsl:value-of select="@from"/>[<xsl:value-of select="@index"/>];
    </xsl:when>
    <xsl:when test="@cmd='ADDNODE'">
        if ( <xsl:value-of select="@node"/> != null )
        {
            <xsl:value-of select="@tolist"/>.Add(<xsl:value-of select="@node"/>);
        }
    </xsl:when>
    <xsl:when test="@cmd='ADDLIST'">
        if ( <xsl:value-of select="@fromlist"/> != null )
        {
            <xsl:value-of select="@tolist"/>.AddAll(<xsl:value-of select="@fromlist"/>);
        }
    </xsl:when>
    <xsl:when test="@cmd='MAKELIST'">
        TypedList <xsl:value-of select="@result"/> = new TypedList();
    </xsl:when>
    <xsl:when test="@cmd='MAKENODE'">
        <xsl:value-of select="@etype"/><xsl:text> </xsl:text><xsl:value-of select="@result"/> = new <xsl:value-of select="@etype"/> <xsl:text>(</xsl:text>
      <xsl:for-each select="arg">
            <xsl:if test="@null">  <!-- empty means it is null -->
              <xsl:value-of select="'null'"/><xsl:if test="position()!=last()">, </xsl:if>
            </xsl:if>
            <xsl:if test="not(@null)">
              <xsl:value-of select="."/><xsl:if test="position()!=last()">, </xsl:if>
            </xsl:if>
      </xsl:for-each>
        <xsl:text>);</xsl:text>
    </xsl:when>
    <xsl:when test="@cmd='RETURNNODE'">
      <xsl:if test="@null">  <!-- empty means it is null -->
        nodeList.Add(null);
      </xsl:if>
      <xsl:if test="not(@null)">
        nodeList.Add(<xsl:value-of select="@node"/>);
      </xsl:if>
    </xsl:when>
    <xsl:when test="@cmd='RETURNLIST'">
        nodeList.Add(<xsl:value-of select="@list"/>);
    </xsl:when>
  </xsl:choose>
</xsl:for-each>
        return nodeList;
    }
</xsl:for-each>

    private static int[][][] actionTable = {
<xsl:for-each select="parser_data/action_table/row">
      <xsl:text>new int[][] {</xsl:text>
      <xsl:for-each select="action">
        <xsl:value-of select="concat('new int[] {', @from, ', ', @action, ', ', @to, '}')"/><xsl:if test="position()!=last()">, </xsl:if>
      </xsl:for-each>
      <xsl:text>}</xsl:text>
      <xsl:if test="position()!=last()">,
      </xsl:if>
</xsl:for-each>
    };

    private static int[][][] gotoTable  = {
<xsl:for-each select="parser_data/goto_table/row">
      <xsl:text>new int [][] {</xsl:text>
      <xsl:for-each select="goto">
        <xsl:value-of select="concat('new int[] {', @from, ', ', @to, '}')"/><xsl:if test="position()!=last()">, </xsl:if>
      </xsl:for-each>
      <xsl:text>}</xsl:text>
      <xsl:if test="position()!=last()">,
      </xsl:if>
</xsl:for-each>
    };

    private static String[] errorMessages = {
<xsl:for-each select="parser_data/error_messages/msg">
      "<xsl:value-of select="sablecc:escape_string_literal(string(.))"/>"<xsl:if test="position()!=last()">, </xsl:if>
</xsl:for-each>
    };

    private static int[] errors = {
<xsl:for-each select="parser_data/errors/i">
    <xsl:value-of select="."/><xsl:if test="position()!=last()">, </xsl:if>
</xsl:for-each>
    };
}
}
    </redirect:write>
  </xsl:template>

</xsl:stylesheet>

