<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:redirect="http://xml.apache.org/xalan/redirect"
    xmlns:sablecc="sablecc-extension"
    extension-element-prefixes="redirect sablecc">

  <xsl:template name="make_prods">

  <!-- the first proudction name that goes into Start -->
  <xsl:param name="baseprod" select="productions/production/@ename"/>

    <redirect:write select="concat($basedir, '/prods.cs')" mkdirs="true">

using System;
using System.Collections;
using System.Text;

using  <xsl:value-of select="/parser/@package"/>.analysis;

namespace <xsl:value-of select="/parser/@package"/>.node {

    <!-- generate all production classes -->
    <xsl:for-each select="productions/production">

public abstract class <xsl:value-of select="@ename"/> : Node
{
}
    </xsl:for-each>

    <!-- generate all alternative classes -->
    <xsl:for-each select="productions/production/alt">

public sealed class <xsl:value-of select="@ename"/> : <xsl:value-of select="../@ename"/>
{
<!-- define subnodes -->
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
    private TypedList _<xsl:value-of select="@name"/>_;
  </xsl:if>

  <xsl:if test="not(@is_list)">
    private <xsl:value-of select="@etype"/> _<xsl:value-of select="@name"/>_;
  </xsl:if>
</xsl:for-each>

<!-- define default empty constructor -->
<xsl:if test="elem">
    public <xsl:value-of select="@ename"/> ()
    {
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
        this._<xsl:value-of select="@name"/>_ = new TypedList(new <xsl:value-of select="@ename"/>_Cast(this));
  </xsl:if>
</xsl:for-each>
    }
</xsl:if>

<!-- define constructor -->
    public <xsl:value-of select="@ename"/> (
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
            IList _<xsl:value-of select="@name"/>_
  </xsl:if>

  <xsl:if test="not(@is_list)">
            <xsl:value-of select="@etype"/> _<xsl:value-of select="@name"/>_
  </xsl:if>
  <xsl:if test="position()!=last()">,</xsl:if>
</xsl:for-each>
    )
    {
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
        this._<xsl:value-of select="@name"/>_ = new TypedList(new <xsl:value-of select="@ename"/>_Cast(this));
        this._<xsl:value-of select="@name"/>_.Clear();
        this._<xsl:value-of select="@name"/>_.AddAll(_<xsl:value-of select="@name"/>_);
  </xsl:if>

  <xsl:if test="not(@is_list)">
        Set<xsl:value-of select="@ename"/> (_<xsl:value-of select="@name"/>_);
  </xsl:if>
</xsl:for-each>
    }

<!-- define clone() -->
    public override Object Clone()
    {
        return new <xsl:value-of select="@ename"/> (
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
            CloneList (_<xsl:value-of select="@name"/>_)
  </xsl:if>

  <xsl:if test="not(@is_list)">
            (<xsl:value-of select="@etype"/>)CloneNode (_<xsl:value-of select="@name"/>_)
  </xsl:if>
  <xsl:if test="position()!=last()">,</xsl:if>
</xsl:for-each>
        );
    }

<!-- define apply() -->
    public override void Apply(Switch sw)
    {
        ((Analysis) sw).Case<xsl:value-of select="@ename"/>(this);
    }

<!-- define accessors -->
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
    public IList Get<xsl:value-of select="@ename"/> ()
    {
        return _<xsl:value-of select="@name"/>_;
    }

    public void set<xsl:value-of select="@ename"/> (IList list)
    {
        _<xsl:value-of select="@name"/>_.Clear();
        _<xsl:value-of select="@name"/>_.AddAll(list);
    }
  </xsl:if>

  <xsl:if test="not(@is_list)">
    public <xsl:value-of select="@etype"/> Get<xsl:value-of select="@ename"/> ()
    {
        return _<xsl:value-of select="@name"/>_;
    }

    public void Set<xsl:value-of select="@ename"/> (<xsl:value-of select="@etype"/> node)
    {
        if(_<xsl:value-of select="@name"/>_ != null)
        {
            _<xsl:value-of select="@name"/>_.Parent(null);
        }

        if(node != null)
        {
            if(node.Parent() != null)
            {
                node.Parent().RemoveChild(node);
            }

            node.Parent(this);
        }

        _<xsl:value-of select="@name"/>_ = node;
    }
  </xsl:if>
</xsl:for-each>

<!-- define ToString() -->
    public override string ToString()
    {
        return ""
<xsl:for-each select="elem">
            + ToString (_<xsl:value-of select="@name"/>_)
</xsl:for-each>
        ;
    }

<!-- define removeChild() -->
    internal override void RemoveChild(Node child)
    {
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
        if ( _<xsl:value-of select="@name"/>_.Contains(child) )
        {
            _<xsl:value-of select="@name"/>_.Remove(child);
            return;
        }
  </xsl:if>

  <xsl:if test="not(@is_list)">
        if ( _<xsl:value-of select="@name"/>_ == child )
        {
            _<xsl:value-of select="@name"/>_ = null;
            return;
        }
  </xsl:if>
</xsl:for-each>
    }

<!-- define replaceChild() -->
    internal override void ReplaceChild(Node oldChild, Node newChild)
    {
<xsl:for-each select="elem">
  <xsl:if test="@is_list">
        for ( int i = 0; i &lt; _<xsl:value-of select="@name"/>_.Count; i++ )
        {
            Node n = (Node)_<xsl:value-of select="@name"/>_[i];
            if(n == oldChild)
            {
                if(newChild != null)
                {
                    _<xsl:value-of select="@name"/>_[i] = newChild;
                    oldChild.Parent(null);
                    return;
                }

                _<xsl:value-of select="@name"/>_.RemoveAt(i);
                oldChild.Parent(null);
                return;
            }
        }
  </xsl:if>

  <xsl:if test="not(@is_list)">
        if ( _<xsl:value-of select="@name"/>_ == oldChild )
        {
            Set<xsl:value-of select="@ename"/> ((<xsl:value-of select="@etype"/>) newChild);
            return;
        }
  </xsl:if>
</xsl:for-each>
    }

<!-- define cast inner classes for any list elements -->
<xsl:for-each select="elem[@is_list]">
    private class <xsl:value-of select="@ename"/>_Cast : Cast
    {
        <xsl:value-of select="../@ename"/> obj;

        internal <xsl:value-of select="@ename"/>_Cast (<xsl:value-of select="../@ename"/> obj)
        {
          this.obj = obj;
        }

        public Object Cast(Object o)
        {
            <xsl:value-of select="@etype"/> node = (<xsl:value-of select="@etype"/>) o;

            if((node.Parent() != null) &amp;&amp;
                (node.Parent() != obj))
            {
                node.Parent().RemoveChild(node);
            }

            if((node.Parent() == null) ||
                (node.Parent() != obj))
            {
                node.Parent(obj);
            }

            return node;
        }
    }
</xsl:for-each>
}
    </xsl:for-each>

    <!-- generate Start class -->

public sealed class Start : Node
{
    private <xsl:value-of select="$baseprod"/> _base_;
    private EOF _eof_;

    public Start()
    {
    }

    public Start(
        <xsl:value-of select="$baseprod"/> _base_,
        EOF _eof_)
    {
        Set<xsl:value-of select="$baseprod"/>(_base_);
        SetEOF(_eof_);
    }

    public override Object Clone()
    {
        return new Start(
            (<xsl:value-of select="$baseprod"/>) CloneNode(_base_),
            (EOF) CloneNode(_eof_));
    }

    public override void Apply(Switch sw)
    {
        ((Analysis) sw).CaseStart(this);
    }

    public <xsl:value-of select="$baseprod"/> Get<xsl:value-of select="$baseprod"/>()
    {
        return _base_;
    }
    public void Set<xsl:value-of select="$baseprod"/>(<xsl:value-of select="$baseprod"/> node)
    {
        if(_base_ != null)
        {
            _base_.Parent(null);
        }

        if(node != null)
        {
            if(node.Parent() != null)
            {
                node.Parent().RemoveChild(node);
            }

            node.Parent(this);
        }

        _base_ = node;
    }

    public EOF GetEOF()
    {
        return _eof_;
    }
    public void SetEOF(EOF node)
    {
        if(_eof_ != null)
        {
            _eof_.Parent(null);
        }

        if(node != null)
        {
            if(node.Parent() != null)
            {
                node.Parent().RemoveChild(node);
            }

            node.Parent(this);
        }

        _eof_ = node;
    }

    internal override void RemoveChild(Node child)
    {
        if(_base_ == child)
        {
            _base_ = null;
            return;
        }

        if(_eof_ == child)
        {
            _eof_ = null;
            return;
        }
    }
    internal override void ReplaceChild(Node oldChild, Node newChild)
    {
        if(_base_ == oldChild)
        {
            Set<xsl:value-of select="$baseprod"/>((<xsl:value-of select="$baseprod"/>) newChild);
            return;
        }

        if(_eof_ == oldChild)
        {
            SetEOF((EOF) newChild);
            return;
        }
    }

    public override string ToString()
    {
        return "" +
            ToString(_base_) +
            ToString(_eof_);
    }
}
}
    </redirect:write>
  </xsl:template>

</xsl:stylesheet>

